;;
;; readline module for Gauche
;;
;; by: YOKOTA Hiroshi

(define-module readline
  (use gauche.uvector)
  (use gauche.vport)
  (use srfi-1)
  (export make-readline-input-port
	  rl-readline
	  rl-set-application-name!
	  rl-add-history! rl-read-history! rl-write-history rl-stifle-history!
	  rl-set-prompt!
	  rl-set-prompt-state!
	  rl-set-completion-list-function!
	  rl-set-basic-word-break-characters!
	  rl-set-basic-quote-characters!
	  rl-commit-history!
	  ))
(select-module readline)

(dynamic-load "readline")

(define (make-readline-input-port)
  (make <buffered-input-port>
    :fill (lambda (vec)
	    (let ((line #f))

	      (set! line (rl-readline rl-prompt))
	      (rl-set-prompt-state! 'ps2)

	      (if (string? line)
		  (let* ((rlvec (string->u8vector line))
			 (len   (u8vector-length  rlvec)))

		    (if (not (= (string-length line) 0))
			(rl-add-history-buffer! line))

		    (u8vector-copy! vec 0 rlvec)

		    ;; Add #\newline for line separator. Then add
		    ;; 1 to length and exits this function with
		    ;; length value.
		    (u8vector-set! vec len (char->integer #\newline))
		    (+ len 1))

		  0)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; history

(define rl-history-buffer '())
(define (rl-add-history-buffer! str)
  (set! rl-history-buffer `(,@rl-history-buffer ,str)))

; stack code segments and make one long code
(define rl-commit-history!
  (let ((prev-cmd '()))
    (lambda ()
      (if (not (null? rl-history-buffer))
	  (begin
	    (if (not (list= string=? prev-cmd rl-history-buffer))
		(rl-add-history! (string-join rl-history-buffer " ")))
	    (set! prev-cmd rl-history-buffer)
	    (set! rl-history-buffer '()))
	    ))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; prompt

; prompt strings
(define rl-prompt  "")
(define rl-prompt1 "readline> ")
(define rl-prompt2 "")

(define (rl-set-prompt! . args)
  (let ((ps1 (get-keyword :ps1 args rl-prompt1))
	(ps2 (get-keyword :ps2 args rl-prompt2)))

    (if (and (string? ps1)
	     (string? ps2))
	(begin
	  (set! rl-prompt1 ps1)
	  (set! rl-prompt2 ps2))
	(error "String required for prompt strings"))))

(define (rl-set-prompt-state! state)
  (if (or (eq? state 'ps1)
	  (eq? state 'ps2))
      (set! rl-prompt
	    (cond ((eq? state 'ps1) rl-prompt1)
		  ((eq? state 'ps2) rl-prompt2)))
      (error "Prompt state must be 'ps1 or 'ps2")))

(provide "readline")

;; end
